@@ -31,8 +31,10 @@ from __future__ import unicode_literals
 import sys
 import logging
 import numbers
-from mappyfile.tokens import COMPOSITE_NAMES, SINGLETON_COMPOSITE_NAMES, REPEATED_KEYS
+from mappyfile.tokens import COMPOSITE_NAMES, SINGLETON_COMPOSITE_NAMES, \
+                             REPEATED_KEYS, COMPLEX_TYPES, OBJECT_LIST_KEYS
 from mappyfile.validator import Validator
+import mappyfile as utils
 
 log = logging.getLogger("mappyfile")
 
@@ -130,7 +132,8 @@ class Quoter(object):
 
 
 class PrettyPrinter(object):
-    def __init__(self, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=False, **kwargs):
+    def __init__(self, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=False,
+                 align_values=False, separate_complex_types=False, **kwargs):
         """
         Option use "\t" for spacer with an indent of 1
         """
@@ -144,6 +147,8 @@ class PrettyPrinter(object):
         self.end_comment = end_comment
         self.end = u"END"
         self.validator = Validator()
+        self.align_values = align_values
+        self.separate_complex_types = separate_complex_types
 
     def __is_metadata(self, key):
         """
@@ -155,6 +160,42 @@ class PrettyPrinter(object):
         else:
             return False
 
+    def compute_aligned_max_indent(self, max_key_length):
+        """
+        Computes the indentation as a multiple of self.indent for aligning
+        values at the same column based on the maximum key length.
+        Example:
+        key         value1
+        longkey     value2
+        longestkey  value3 <-- column at 12, indent of 4, determined by "longestkey"
+        """
+        indent = max(1, self.indent)
+        return int((int(max_key_length / indent) + 1) * indent)
+
+    def compute_max_key_length(self, composite):
+        """
+        Computes the maximum length of all keys (non-recursive) in the passed
+        composite.
+        """
+        length = 0
+        for attr, value in composite.items():
+            attr_length = len(attr)
+            if (not self.__is_metadata(attr) and
+                attr not in ("metadata", "validation", "values", "connectionoptions") and
+                not self.is_hidden_container(attr, value) and not attr == "pattern" and
+                not attr == "projection" and not attr == "points" and
+                    not attr == "config" and not self.is_composite(value)):
+                length = max(length, attr_length)
+
+        return length
+
+    def separate_complex(self, composite, level):
+        if not self.separate_complex_types:
+            return
+        for key in list(composite.keys()):
+            if self.is_complex_type(composite, key, level):
+                utils.dict_move_to_end(composite, key)
+
     def whitespace(self, level, indent):
         return self.spacer * (level + indent)
 
@@ -167,13 +208,16 @@ class PrettyPrinter(object):
             end_line = "{} # {}".format(end_line, key.upper())
         return end_line
 
-    def __format_line(self, spacer, key, value):
-
-        tmpl = u"{spacer}{key} {value}"
+    def __format_line(self, spacer, key, value, aligned_max_indent=0):
+        if ((aligned_max_indent is None) or (aligned_max_indent == 0)):
+            aligned_max_indent = len(key) + 1
+        indent = " " * (aligned_max_indent - len(key))
+        tmpl = u"{spacer}{key}{indent}{value}"
         d = {
             "spacer": spacer,
             "key": key,
-            "value": value
+            "value": value,
+            "indent": indent
         }
         return tmpl.format(**d)
 
@@ -199,11 +243,16 @@ class PrettyPrinter(object):
         """
         lines = []
 
+        aligned_max_indent = 0
+        if (self.align_values):
+            max_key_length = self.compute_max_key_length(d) + 2  # add length of quotes
+            aligned_max_indent = self.compute_aligned_max_indent(max_key_length)
+
         for k, v in d.items():
             if not self.__is_metadata(k):
                 qk = self.quoter.add_quotes(k)
                 qv = self.quoter.add_quotes(v)
-                line = self.__format_line(self.whitespace(level, 2), qk, qv)
+                line = self.__format_line(self.whitespace(level, 2), qk, qv, aligned_max_indent)
                 line += self.process_attribute_comment(comments, k)
                 lines.append(line)
 
@@ -220,7 +269,7 @@ class PrettyPrinter(object):
             lines.append(self.__format_line(self.whitespace(level, 1), k, v))
         return lines
 
-    def process_repeated_list(self, key, lst, level):
+    def process_repeated_list(self, key, lst, level, aligned_max_indent=1):
         """
         Process blocks of repeated keys e.g. FORMATOPTION
         """
@@ -229,7 +278,7 @@ class PrettyPrinter(object):
         for v in lst:
             k = key.upper()
             v = self.quoter.add_quotes(v)
-            lines.append(self.__format_line(self.whitespace(level, 1), k, v))
+            lines.append(self.__format_line(self.whitespace(level, 1), k, v, aligned_max_indent))
 
         return lines
 
@@ -292,15 +341,19 @@ class PrettyPrinter(object):
         else:
             return False
 
+    def is_complex_type(self, composite, key, level):
+        # symbol needs special treatment
+        if key == "symbol" and level > 0:
+            return False
+        return key in COMPLEX_TYPES or self.is_composite(key) or self.is_hidden_container(key, composite[key])
+
     def is_hidden_container(self, key, val):
         """
         The key is not one of the Mapfile keywords, and its
         values are a list
         """
 
-        if key in ("layers", "classes", "styles", "symbols", "labels",
-                   "outputformats", "features", "scaletokens",
-                   "composites", "joins") and isinstance(val, list):
+        if key in OBJECT_LIST_KEYS and isinstance(val, list):
             return True
         else:
             return False
@@ -373,8 +426,15 @@ class PrettyPrinter(object):
             return str(value).upper()
 
         if any(i in ["enum"] for i in attr_props):
+
+            if isinstance(value, dict) and not value:
+                raise ValueError("The property {} has an empty dictionary as a value".format(attr))
+
             if not isinstance(value, numbers.Number):
-                return value.upper()  # value is from a set list, no need for quote
+                if attr == "compop":
+                    return self.quoter.add_quotes(value)
+                else:
+                    return value.upper()  # value is from a set list, no need for quote
             else:
                 return value
 
@@ -423,15 +483,14 @@ class PrettyPrinter(object):
 
         return value
 
-    def process_attribute(self, type_, attr, value, level):
+    def process_attribute(self, type_, attr, value, level, aligned_max_indent=1):
         """
         Process one of the main composite types (see the type_ value)
         """
 
         attr_props = self.get_attribute_properties(type_, attr)
         value = self.format_value(attr, attr_props, value)
-        line = self.__format_line(self.whitespace(level, 1), attr.upper(), value)
-
+        line = self.__format_line(self.whitespace(level, 1), attr.upper(), value, aligned_max_indent)
         return line
 
     def format_comment(self, spacer, value):
@@ -494,6 +553,13 @@ class PrettyPrinter(object):
             s = self.whitespace(level, 0) + type_.upper()
             lines.append(s)
 
+        aligned_max_indent = 0
+        if self.align_values:
+            max_key_length = self.compute_max_key_length(composite)
+            aligned_max_indent = self.compute_aligned_max_indent(max_key_length)
+
+        self.separate_complex(composite, level)
+
         for attr, value in composite.items():
             if self.__is_metadata(attr):
                 # skip hidden attributes
@@ -512,7 +578,7 @@ class PrettyPrinter(object):
             elif attr == "projection":
                 lines += self.process_projection(attr, value, level)
             elif attr in REPEATED_KEYS:
-                lines += self.process_repeated_list(attr, value, level)
+                lines += self.process_repeated_list(attr, value, level, aligned_max_indent)
             elif attr == "points":
                 lines += self.format_repeated_pair_list(attr, value, level)
             elif attr == "config":
@@ -523,7 +589,7 @@ class PrettyPrinter(object):
                 # standard key value pair
                 if not type_:
                     raise UnboundLocalError("The Mapfile object is missing a __type__ attribute")
-                line = self.process_attribute(type_, attr, value, level)
+                line = self.process_attribute(type_, attr, value, level, aligned_max_indent)
                 line += self.process_attribute_comment(comments, attr)
                 lines.append(line)
 
