@@ -2,7 +2,7 @@
 #
 # Authors: Seth Girvin
 #
-# Copyright (c) 2020 Seth Girvin
+# Copyright (c) 2021 Seth Girvin
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -28,9 +28,11 @@
 # =================================================================
 
 from __future__ import unicode_literals
+import sys
 import codecs
 import warnings
 import functools
+from collections import OrderedDict
 from mappyfile.parser import Parser
 from mappyfile.transformer import MapfileToDict
 from mappyfile.pprint import PrettyPrinter
@@ -190,7 +192,8 @@ def loads(s, expand_includes=True, include_position=False, include_comments=Fals
     return d
 
 
-def dump(d, fp, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=False):
+def dump(d, fp, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=False,
+         align_values=False, separate_complex_types=False):
     """
     Write d (the Mapfile dictionary) as a formatted stream to fp
 
@@ -213,6 +216,12 @@ def dump(d, fp, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=F
     end_comment: bool
         Add a comment with the block type at each closing END
         statement e.g. END # MAP
+    align_values: bool
+        Aligns the values in the same column for better readability. The column is
+        multiple of indent and determined by the longest key
+    separate_complex_types: bool
+        Groups composites (complex mapserver definitions with "END") together at the end.
+        Keeps the given order except that all simple key-value pairs appear before composites.
 
     Example
     -------
@@ -227,11 +236,12 @@ def dump(d, fp, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=F
             mappyfile.dump(d, f, indent=2, quote="'")
 
     """
-    map_string = _pprint(d, indent, spacer, quote, newlinechar, end_comment)
+    map_string = _pprint(d, indent, spacer, quote, newlinechar, end_comment, align_values, separate_complex_types)
     fp.write(map_string)
 
 
-def save(d, output_file, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=False, **kwargs):
+def save(d, output_file, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=False,
+         align_values=False, separate_complex_types=False, **kwargs):
     """
     Write a dictionary to an output Mapfile on disk
 
@@ -254,6 +264,12 @@ def save(d, output_file, indent=4, spacer=" ", quote='"', newlinechar="\n", end_
     end_comment: bool
         Add a comment with the block type at each closing END
         statement e.g. END # MAP
+    align_values: bool
+        Aligns the values in the same column for better readability. The column is
+        multiple of indent and determined by the longest key.
+    separate_complex_types: bool
+        Groups composites (complex mapserver definitions with "END") together at the end.
+        Keeps the given order except that all simple key-value pairs appear before composites.
 
     Returns
     -------
@@ -272,12 +288,13 @@ def save(d, output_file, indent=4, spacer=" ", quote='"', newlinechar="\n", end_
         fn = "C:/Data/mymap.map"
         mappyfile.save(d, fn)
     """
-    map_string = _pprint(d, indent, spacer, quote, newlinechar, end_comment)
+    map_string = _pprint(d, indent, spacer, quote, newlinechar, end_comment, align_values, separate_complex_types)
     _save(output_file, map_string)
     return output_file
 
 
-def dumps(d, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=False, **kwargs):
+def dumps(d, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=False,
+          align_values=False, separate_complex_types=False, **kwargs):
     """
     Output a Mapfile dictionary as a string
 
@@ -298,6 +315,12 @@ def dumps(d, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=Fals
     end_comment: bool
         Add a comment with the block type at each closing END
         statement e.g. END # MAP
+    align_values: bool
+        Aligns the values in the same column for better readability. The column is
+        multiple of indent and determined by the longest key
+    separate_complex_types: bool
+        Groups composites (complex mapserver definitions with "END") together at the end.
+        Keeps the given order except that all simple key-value pairs appear before composites.
 
     Returns
     -------
@@ -316,7 +339,7 @@ def dumps(d, indent=4, spacer=" ", quote='"', newlinechar="\n", end_comment=Fals
         d = mappyfile.loads(s)
         print(mappyfile.dumps(d, indent=1, spacer="\\t"))
     """
-    return _pprint(d, indent, spacer, quote, newlinechar, end_comment, **kwargs)
+    return _pprint(d, indent, spacer, quote, newlinechar, end_comment, align_values, separate_complex_types, **kwargs)
 
 
 def find(lst, key, value):
@@ -620,8 +643,58 @@ def _save(output_file, string):
         f.write(string)
 
 
-def _pprint(d, indent, spacer, quote, newlinechar, end_comment, **kwargs):
+def _pprint(d, indent, spacer, quote, newlinechar, end_comment, align_values, separate_complex_types, **kwargs):
     pp = PrettyPrinter(indent=indent, spacer=spacer,
                        quote=quote, newlinechar=newlinechar,
-                       end_comment=end_comment, **kwargs)
+                       end_comment=end_comment, align_values=align_values,
+                       separate_complex_types=separate_complex_types, **kwargs)
     return pp.pprint(d)
+
+
+def create(type, version=None):
+    """
+    Create a new mappyfile object, using MapServer defaults (if any).
+
+    Parameters
+    ----------
+
+    s: type
+        The mappyfile type to be stored in the __type__ property
+
+    Returns
+    -------
+
+    dict
+        A Python dictionary representing the Mapfile object in the mappyfile format
+    """
+
+    # get the schema for this type
+
+    v = Validator()
+    try:
+        schema = v.get_versioned_schema(version=version, schema_name=type)
+    except IOError:
+        raise SyntaxError("The mappyfile type '{}' does not exist!".format(type))
+
+    d = OrderedDict()
+    d["__type__"] = type
+
+    properties = sorted(schema["properties"].items())
+
+    for k, v in properties:
+        if "default" in v:
+            d[k] = v["default"]
+
+    return d
+
+
+def dict_move_to_end(ordered_dict, key):
+
+    if sys.version_info[0] < 3:
+        # mappyfile requires Python >= 2.7,
+        # so this should be safe
+        val = ordered_dict[key]
+        del ordered_dict[key]
+        ordered_dict[key] = val
+    else:
+        ordered_dict.move_to_end(key)
