@@ -12,6 +12,7 @@ import pytest
 from mappyfile.parser import Parser
 from mappyfile.pprint import PrettyPrinter, Quoter
 from mappyfile.transformer import MapfileToDict
+from mappyfile.ordereddict import DefaultOrderedDict
 import mappyfile
 
 
@@ -430,6 +431,78 @@ END # MAP"""
     assert res == exp
 
 
+def test_empty_composite():
+    s = "MAP LAYER TYPE POINT NAME 'Test' END END"
+    ast = mappyfile.loads(s)
+    lyr = ast["layers"][0]
+
+    # as we are using DefaultOrderedDict then checks like this example
+    # will create an empty dictionary object
+    if lyr["connectiontype"]:
+        pass
+
+    assert isinstance(lyr["connectiontype"], DefaultOrderedDict)
+
+    pp = PrettyPrinter(indent=4, quote='"', newlinechar="\n", end_comment=True)
+    error_thrown = False
+
+    try:
+        pp.pprint(ast)
+    except ValueError:
+        error_thrown = True
+
+    assert error_thrown
+
+
+def test_align_values():
+    mapfile = """
+    MAP
+        EXTENT 0 0 100 100
+        DEBUG on
+        NAME Test
+        shapepath "test/path"
+    END
+    """
+    ast = mappyfile.loads(mapfile)
+    pp = PrettyPrinter(indent=0, quote="'", newlinechar="\n", align_values=True)
+    s = pp.pprint(ast)
+
+    aligned_indexes = set()
+    for line in s.splitlines():
+        words = line.split()
+        if len(words) > 1:
+            aligned_indexes.add(line.index(words[1]))
+    assert len(aligned_indexes) == 1
+
+
+def test_separate_complex_types():
+    s = """
+    MAP
+        EXTENT 0 0 100 100
+        shapepath "test/path"
+        LAYER
+            CLASS
+                EXPRESSION "Field"
+                STYLE
+                    SIZE [sizefield]
+                END
+                GROUP 'group1'
+            END
+            CLASSITEM "Test"
+        END
+        DEBUG on
+        NAME Test
+    END
+    """
+    ast = mappyfile.loads(s)
+    pp = PrettyPrinter(indent=4, quote="'", newlinechar="\n", separate_complex_types=True)
+    s = pp.pprint(ast)
+    assert s.index("NAME ") < s.index("LAYER\n")
+    assert s.index("DEBUG ") < s.index("LAYER\n")
+    assert s.index("CLASSITEM ") < s.index("CLASS\n")
+    assert s.index("GROUP") < s.index("STYLE\n")
+
+
 def run_tests():
     # pytest.main(["tests/test_pprint.py::test_format_list"])
     pytest.main(["tests/test_pprint.py"])
@@ -437,6 +510,6 @@ def run_tests():
 
 if __name__ == "__main__":
     logging.basicConfig(level=logging.INFO)
-    test_connectionoptions()
+    test_empty_composite()
     # run_tests()
     print("Done!")
