@@ -916,13 +916,14 @@ def test_multiple_layer_data():
 
     s = u"""
     LAYER
+        TYPE POINT
         DATA "dataset1"
         DATA "dataset2"
     END
     """
 
     print(output(s, schema_name="layer"))
-    exp = u"LAYER DATA 'dataset1' DATA 'dataset2' END"
+    exp = u"LAYER TYPE POINT DATA 'dataset1' DATA 'dataset2' END"
     assert(output(s, schema_name="layer") == exp)
 
 
@@ -930,6 +931,7 @@ def test_single_layer_data():
 
     s = u"""
     LAYER
+        TYPE POINT
         DATA "dataset1"
     END
     """
@@ -939,7 +941,7 @@ def test_single_layer_data():
     print(mappyfile.dumps(jsn))
 
     print(output(s, schema_name="layer"))
-    exp = u"LAYER DATA 'dataset1' END"
+    exp = u"LAYER TYPE POINT DATA 'dataset1' END"
     assert(output(s, schema_name="layer") == exp)
 
 
@@ -947,6 +949,7 @@ def test_cluster():
 
     s = u"""
     LAYER
+        TYPE POINT
         CLUSTER
             MAXDISTANCE 50
             REGION "ELLIPSE"
@@ -954,7 +957,7 @@ def test_cluster():
     END
     """
     print(output(s, schema_name="layer"))
-    exp = u"LAYER CLUSTER MAXDISTANCE 50 REGION 'ELLIPSE' END END"
+    exp = u"LAYER TYPE POINT CLUSTER MAXDISTANCE 50 REGION 'ELLIPSE' END END"
     assert(output(s, schema_name="layer") == exp)
 
 
@@ -966,6 +969,7 @@ def test_cluster2():
     """
     s = u"""
     LAYER
+        TYPE POINT
         CLUSTER
             MAXDISTANCE 50
             REGION ELLIPSE
@@ -973,7 +977,7 @@ def test_cluster2():
     END
     """
     print(output(s, schema_name="layer"))
-    exp = u"LAYER CLUSTER MAXDISTANCE 50 REGION 'ELLIPSE' END END"
+    exp = u"LAYER TYPE POINT CLUSTER MAXDISTANCE 50 REGION 'ELLIPSE' END END"
     assert(output(s, schema_name="layer") == exp)
 
 
@@ -994,6 +998,34 @@ def test_outputformat_unquoted_keyword():
     assert(output(s, schema_name="map") == exp)
 
 
+def test_multiple_compfilters():
+    """
+    See https://github.com/geographika/mappyfile/issues/150
+    """
+
+    s = u"""
+    LAYER
+        NAME "point-symbol-test"
+        TYPE POINT
+        COMPOSITE
+            COMPFILTER "blacken()"
+            COMPFILTER "translate(-6,-5)"
+            COMPFILTER "blur(7)"
+            COMPOP "soft-light"
+            OPACITY 50
+        END
+        COMPOSITE
+            OPACITY 100
+        END
+    END
+    """
+
+    print(output(s, schema_name="layer"))
+    exp = u"LAYER NAME 'point-symbol-test' TYPE POINT COMPOSITE COMPFILTER 'blacken()' " \
+    "COMPFILTER 'translate(-6,-5)' COMPFILTER 'blur(7)' COMPOP 'soft-light' OPACITY 50 END COMPOSITE OPACITY 100 END END"
+    assert(output(s, schema_name="layer") == exp)
+
+
 def run_tests():
     r"""
     Need to comment out the following line in C:\VirtualEnvs\mappyfile\Lib\site-packages\pep8.py
@@ -1006,6 +1038,6 @@ def run_tests():
 
 if __name__ == '__main__':
     logging.basicConfig(level=logging.DEBUG)
-    test_label_attribute_properties()
+    test_multiple_compfilters()
     # run_tests()
     print("Done!")
